'''Copyright Tony Valier-Brasier 2024
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License
along with this program. If not, see https://www.gnu.org/licenses/.'''

import numpy as np
from matplotlib import pyplot as plt
import ScatteringCoefficients as fc_CD
import MST
import Functions
params = {'mathtext.fontset': 'cm', 'mathtext.rm': 'serif', 'font.size' : 15, 'legend.fontsize': 15,
          'xtick.labelsize': 15, 'ytick.labelsize': 15}
plt.rcParams.update(params)


''' Elastic parameters '''
R = 500e-9 # Sphere radius
f = 10**np.linspace(0,2.2,201)*1e6 # Frequency
mat_m = Functions.Materiau('eau',f) # Parameters of water
mat_p = Functions.Materiau('silica')# Parameters of spheres
kL = 2*np.pi*f*np.sqrt(mat_m.rho/(mat_m.lam + 2*mat_m.mu)) # Longitudinal wavenumber in water
kT = 2*np.pi*f*np.sqrt(mat_m.rho/mat_m.mu) # Shear wavenumber in water


''' Calculation of the effective wavenumbers
Tn : scattering coefficients
K_MST : effective wavenumber with or without conversion (conv) and without correlation (corr)

'''
K_MST = np.zeros((6,np.size(f)), dtype = np.complex128)
Tn = fc_CD.Solide_Sphere_elastique(2, f, R, mat_p.lam, mat_p.mu, mat_p.rho, mat_m.lam, mat_m.mu, mat_m.rho)
# Calculations for phi = 10%
phi = 0.1 # Concentration
K_MST[0,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'no')
K_MST[1,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R)
phi = 0.2 # Concentration
K_MST[2,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'no')
K_MST[3,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R)
phi = 0.3 # Concentration
K_MST[4,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'no')
K_MST[5,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R)


''' PLOTS OF THE EFFECTIVE ATTENUATION '''
Attenuation = np.imag(K_MST)
lab = ['$\\phi = 30\%$ Model C', '$\\phi = 30\%$ Model G', '$\\phi = 20\%$ Model C', '$\\phi = 20\%$ Model G', '$\\phi = 10\%$ Model C', '$\\phi = 10\%$ Model G']
col = ['k', 'k--', 'r', 'r--', 'C0', 'C0--']
plt.figure(figsize = (11,5))
for nn in range(6):
    plt.semilogx(np.real(kT*R),Attenuation[nn,:]*mat_m.c0/f, col[nn], label = lab[nn])
plt.legend(frameon = False, ncol = 3)
plt.xlim(1,10)
plt.xlabel('Re[$k_Sa$]')
plt.ylabel('$\\alpha_{eff}\\lambda$')
plt.tight_layout()
plt.savefig('Figure2.pdf', transparent = 'true', bbox_inches='tight')
plt.show()
