'''Copyright Tony Valier-Brasier 2024
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License
along with this program. If not, see https://www.gnu.org/licenses/.'''

import numpy as np
import h5py as h5
from matplotlib import pyplot as plt
import ScatteringCoefficients as fc_CD
import MST
import Functions
params = {'mathtext.fontset': 'cm', 'mathtext.rm': 'serif', 'font.size' : 15, 'legend.fontsize': 15,
          'xtick.labelsize': 15, 'ytick.labelsize': 15}
plt.rcParams.update(params)


''' Elastic parameters '''
R = 500e-9 # Sphere radius
f = 10**np.linspace(0,2.2,201)*1e6 # Frequency
mat_m = Functions.Materiau('eau',f) # Parameters of water
mat_p = Functions.Materiau('silica')# Parameters of spheres
kL = 2*np.pi*f*np.sqrt(mat_m.rho/(mat_m.lam + 2*mat_m.mu)) # Longitudinal wavenumber in water
kT = 2*np.pi*f*np.sqrt(mat_m.rho/mat_m.mu) # Shear wavenumber in water

''' Pair correlation functions '''
data_file = h5.File('Spheres_phi20_a500nm.h5', 'r')
r20 = data_file['r'][()]
g2_20 = data_file['g2_mean'][()]
data_file.close()
data_file = h5.File('Spheres_phi30_a500nm.h5', 'r')
r30 = data_file['r'][()]
g2_30 = data_file['g2_mean'][()]
data_file.close()
data_file = h5.File('Spheres_phi40_a500nm.h5', 'r')
r40 = data_file['r'][()]
g2_40 = data_file['g2_mean'][()]
data_file.close()


''' Calculation of the effective wavenumbers
Tn : scattering coefficients
K_MST : effective wavenumber with or without conversion (conv) and without correlation (corr)

'''
K_MST = np.zeros((6,np.size(f)), dtype = np.complex128)
Tn = fc_CD.Solide_Sphere_elastique(2, f, R, mat_p.lam, mat_p.mu, mat_p.rho, mat_m.lam, mat_m.mu, mat_m.rho)
phi = 0.4 # Concentration
K_MST[0,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'yes',r=r20,g2=g2_20)
K_MST[1,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R,r=r20,g2=g2_20)
phi = 0.3 # Concentration
K_MST[2,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'yes',r=r30,g2=g2_30)
K_MST[3,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R,r=r30,g2=g2_30)
phi = 0.2 # Concentration
K_MST[4,:] = MST.MST_keff(phi,R,kL,kT,Tn,2*R,conv = 'yes',corr = 'yes',r=r40,g2=g2_40)
K_MST[5,:] = MST.MST_exact(phi,R,kL,kT,Tn,2*R,r=r40,g2=g2_40)


''' PLOTS OF THE EFFECTIVE ATTENUATION '''
Attenuation = np.imag(K_MST)
lab = ['$\\phi = 40\%$ Model I', '$\\phi = 40\%$ Model J', '$\\phi = 30\%$ Model I', '$\\phi = 30\%$ Model J', '$\\phi = 20\%$ Model I', '$\\phi = 20\%$ Model J']
col = ['C0', 'C0--', 'k', 'k--', 'r', 'r--']
plt.figure(figsize = (8,5))
for nn in range(6):
    plt.semilogx(np.real(kT*R),Attenuation[nn,:]*mat_m.c0/f, col[nn], label = lab[nn])
plt.legend(frameon = False)
plt.xlim(1,10)
plt.xlabel('Re[$k_Sa$]')
plt.ylabel('$\\alpha_{eff}\\lambda$')
plt.tight_layout()
plt.savefig('Figure5.pdf', transparent = 'true', bbox_inches='tight')
plt.show()
