'''Copyright Tony Valier-Brasier 2024
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License
along with this program. If not, see https://www.gnu.org/licenses/.'''

import numpy as np
import scipy.special as scp
from sympy.physics.wigner import wigner_3j

class Materiau:
    def __init__(self, name, f=0, T_ref=0): 
        self.name = name
        if self.name == 'eau':
            omega = 2*np.pi*f
            self.rho = 997.0
            self.c0 = 1497.0
            self.eta_v = 2.7e-3
            self.eta_s = 0.891e-3
            self.lam = self.rho*self.c0**2 - 1j*omega*(self.eta_v*2/3.0 + self.eta_s)
            self.mu = - 1j*omega*self.eta_s

        elif self.name == 'silica':
            omega = 2*np.pi*f
            self.rho = 2041.0
            self.lam = 10.89e9
            self.mu = 30.9e9

        elif self.name == 'silica_430':
            omega = 2*np.pi*f
            self.rho = 1986.0
            self.lam = 10.89e9
            self.mu = 30.9e9

def gaunt_2D(N_m):
    G = np.zeros((N_m,N_m,2*N_m))
    for n in range(N_m):
        for nu in range(N_m):
            for p in range(2*N_m):
                tmp = wigner_3j(n,nu,p,0,0,0)
                G[n,nu,p] = (2*p + 1.0)*tmp**2
    return G


def def_G(Nm):
    GLL = np.zeros((2,Nm,2*Nm,Nm),dtype = np.complex128)
    GTT = np.zeros((2,Nm,2*Nm,Nm),dtype = np.complex128)
    GTS = np.zeros((2,Nm,2*Nm,Nm),dtype = np.complex128)
    for nu in range(Nm):
        for nn in range(Nm):
            for pp in range(2*Nm):
                G0 = (2*pp + 1)*(wigner_3j(nu,nn,pp,0,0,0))**2
                if nn == 0:
                    a = 0
                    b = 0
                    G1 = 0
                    G2 = 0
                else:               
                    G1 = (2*pp + 1)*np.sqrt(nu*(nu+1)/(nn*(nn+1)))*wigner_3j(nu,nn,pp,0,0,0)*wigner_3j(nu,nn,pp,1,-1,0)
                    G2 = (2*pp + 1)*np.sqrt(nu*(nu+1)/(nn*(nn+1)))*wigner_3j(nu,nn,pp-1,0,0,0)*wigner_3j(nu,nn,pp,1,-1,0)
                    a = 2*nn*(nn + 1)*(2*nn + 1)
                    a += (nn + 1)*(nu - nn + pp + 1)*(nu + nn - pp)
                    a += - nn*(nn - nu + pp + 1)*(nu + nn + pp + 2)
                    a = 1j**(nn + pp - nu)*a/(2*nn*(nn+1))
                    b = (pp**2 - (nn - nu)**2)*((nu + nn + 1)**2 - pp**2)
                    b = 1j**(nn+pp-nu)*np.sqrt(b + 0j)*(2*nn+1)/(2*nn*(nn+1))
                GLL[0,nn,pp,nu] = 1j**(nn - nu + pp)*(2*nn+1)*G0
                GTT[0,nn,pp,nu] = G0*a
                GLL[1,nn,pp,nu] = - 1j**(nn - nu + pp)*(2*nn+1)*G1
                GTT[1,nn,pp,nu] = - G1*a
                GTS[1,nn,pp,nu] = - G2*b
    return GLL,GTT,GTS

def Bessel(n,x,Option = 'No'):
    N_x = np.size(x)
    J = np.zeros((n,N_x),dtype=np.complex128)

    if Option == 'BF':
        J[0,:] = 1 - x**2/6 + x**4/120
        if n == 1:
            J[1,:] = x/3 - x**3/30
        elif n == 2:
            J[1,:] = x/3 - x**3/30
            J[2,:] = x**2/15 - x**4/210
        else:
            J[1,:] = x/3 - x**3/30
            J[2,:] = x**2/15 - x**4/210
            p = 2
            while p < (n-1):
                p = p + 1
                J[p,:] = (2*p - 1)*J[p - 1,:]/x - J[p - 2,:]
    elif Option == 'Normalized':
        J[0,:] = np.exp(2j*x)/2j - 1/2j
        if n == 1:
            J[1,:] = - 1/(2j*x) - 1/2 + np.exp(2j*x)*(1/(2j*x) - 1/2)
        elif n == 2:
            J[1,:] = - 1/(2j*x) - 1/2 + np.exp(2j*x)*(1/(2j*x) - 1/2)
            J[2,:] = - 3/(2*x**4) + 1/(2*x**2) + 3/2j + np.exp(2j*x)*( - 3/(2*x**4) + 1/(2*x**2) - 3/2j)
        else:
            J[1,:] = - 1/(2j*x) - 1/2 + np.exp(2j*x)*(1/(2j*x) - 1/2)
            J[2,:] = - 3/(2*x**4) + 1/(2*x**2) + 3/2j + np.exp(2j*x)*( - 3/(2*x**4) + 1/(2*x**2) - 3/2j)
            p = 2
            while p < (n-1):
                p = p + 1
                J[p,:] = (2*p - 1)*J[p - 1,:]/x - J[p - 2,:]
    else:
        for nu in range(n):
            J[nu,:] = scp.jn(nu + 0.5, x)*np.sqrt(np.pi/(2*x))
    return J


def Hankel(n,x,Option = 'No'):
    N_x = np.size(x)
    H = np.zeros((n,N_x),dtype=np.complex128)

    if Option == 'BF':
        H[0,:] = - 1j/x + 1 + 1j*x/2
        if n == 1:
            H[1,:] = - 1j/x**2 - 1j/2 + x/3
        elif n == 2:
            H[1,:] = - 1j/x**2 - 1j/2 + x/3
            H[2,:] = - 3j/x**3 - 1j/(2*x)
        else:
            H[1,:] = - 1j/x**2 - 1j/2 + x/3
            H[2,:] = - 3j/x**3 - 1j/(2*x)
            p = 2
            while p < (n-1):
                p = p + 1
                H[p,:] = (2*p - 1)*H[p - 1,:]/x - H[p - 2,:]
    elif Option == 'Normalized':
        H[0,:] = - 1j
        if n == 1:
            H[1,:] = - 1 - 1j/x
        elif n == 2:
            H[1,:] = - 1 - 1j/x
            H[2,:] = 1j - 3/x - 3j/x**2
        else:
            H[1,:] = - 1 - 1j/x
            H[2,:] = 1j - 3/x - 3j/x**2
            p = 2
            while p < (n-1):
                p = p + 1
                H[p,:] = (2*p - 1)*H[p - 1,:]/x - H[p - 2,:]
    else:
        for nu in range(n):
            H[nu,:] = scp.hankel1(nu + 0.5, x)*np.sqrt(np.pi/(2*x))
    return H


def d_Bessel(Z,x):
    N = np.size(Z[:,0]) - 1
    dZ = np.zeros((N,np.size(Z[0,:])),dtype=np.complex128)
    for n in range(N):
        dZ[n,:] = n*Z[n,:]/x - Z[n + 1,:]
    return dZ
