function abs_det_M=detDispReln(KEff_ReIm,K,S,fac_C,fac_S,dispRelnCoeffs,ai,aj,exclFac)
% This function provides the objective function 
% (absolute value of determinant of the dispersion relation matrix) 
% for the solution of the full dispersion relation for effective wavenumber
% for a 2-mode system (compressional, shear)
% with KEff near to K or S
% based on Luppe Conoir Valier Brasier 2022 Wave Motion Eq 56
% Inputs:
% KEff: the effective wavenumber (as a matrix of real and imaginary part) near to K or S
% K: the wavenumber of the mode for which an effective quasi-p mode is
% sought
% S: the wavenumber of the mode for which a different effective wavenumber
% for quasi-S mode near to S is sought
% fac_C,S: the volume fraction/(K or S *ai)^3 where ai is scatterer radius
% dispRelnCoeffs: a matrix which includes the factors in the dispersion 
% relation matrix that are independent of the effective wavenumber 
% - these include the factors of the scattering coefficients (Tn_pq) 
% dispRelnCoeffs=[-T0 1i*T1 1i*T1SC; -3i*T0 -T1 -T1SC; 0 -T1CS -T1SS]
% multiMode: integer switch =0 for compressional only (LB), 1 for multiMode
% Neglect the correlation contributions to the function N~ (Eq 22 LCVB22)
% Includes terms up to and including dipole order

sizeMatrix=size(dispRelnCoeffs);

KEff=complex(KEff_ReIm(1),KEff_ReIm(2));
b=exclFac*(ai+aj);%the exclusion zone radius
Kb=K*b;
Sb=S*b;

delB2_K2=KEff*KEff/(K*K)-1;
delB2_S2=KEff*KEff/(S*S)-1;

xi_C=3*fac_C*Kb/delB2_K2;
xi_S=3*fac_S*Sb/delB2_S2;
%Neglect the correlation contributions to the function N~ (Eq 22 LCVB22)
N0_C=multiModeFac(0,KEff,K,ai,aj);
N1_C=multiModeFac(1,KEff,K,ai,aj);
N2_C=multiModeFac(2,KEff,K,ai,aj);
N3_C=multiModeFac(3,KEff,K,ai,aj);
N0_S=multiModeFac(0,KEff,S,ai,aj);
N1_S=multiModeFac(1,KEff,S,ai,aj);
N2_S=multiModeFac(2,KEff,S,ai,aj);
N3_S=multiModeFac(3,KEff,S,ai,aj);

%including up to A2_S coefficients but neglecting T2 coefficients
Z=[xi_C*N0_C xi_C*N1_C xi_C*N1_C xi_C*N2_C;...
    xi_C*N1_C xi_C*(N0_C+2*N2_C) xi_C*(N0_C+2*N2_C) xi_C*(2*N1_C+3*N3_C);...
    0 xi_S*(N0_S-N2_S) xi_S*(N0_S-N2_S) xi_S*(2*N1_S+3*N3_S);...
    0 xi_S*(2*N1_S+3*N3_S) xi_S*(2*N1_S+3*N3_S) xi_S*(N0_S-(10/7)*N2_S)];

M=eye(sizeMatrix)+Z(1:sizeMatrix(1),1:sizeMatrix(2)).*dispRelnCoeffs; %for efficiency, reconstruct the matrix for each trial value while keeping unchanged parts fixed

%The function returns the determinant of the dispersion relation matrix
% for use by an optimiser searching for minimum abs(determinant)
abs_det_M=abs(det(M(1:sizeMatrix(1),1:sizeMatrix(2))));

end