function [KEff,vel,att]=multScattSolnMultiMode(nMax,radiusType,volFract,freq,Kf,Lf,Sf,TT,TTP,exclFac,numModels,multiModeOptions)
% Multiple scattering calculations for the effective wavenumber,
% effective wave speed and attenuation for
% 3D system of scatterers with scattering coefficients provided
% Monodisperse, single species only
% Multi mode scattering based on
% Luppe Conoir Valier-Brasier 2022
% and asymptotic analysis by Valerie Pinfield published in
% Pinfield and Valier-Braiser 2024, Proceedings of the Royal Society A
% Kf,Lf,Sf are the set of wavenumbers in continuous phase for
% compressional, thermal and shear modes
% radiusType(numSubs,numDist) in metres
% volFract(numSubs, numVol)
% TT and TTP are of shape: numSubs,numModes,numFreq,numSizes,numOrders)
% They store the scattering coefficients (T matrix) for incident C and
% incident shear waves respectively (NB for n=0 TTP stores incident thermal)
% This code assumes that the scattering coefficients are stored in their
% scaled form: Tn_CC; Tn_Cj*hn(kja); Tn_jC/jn(kja); Tn_jj*hn(kja)/jn(kja)
% where j represents the other mode (thermal or shear), n the order
% exclFac: the factor determining the radius of the excluded volume
% =2*exclFac*scatterer radius
% multiModeOptions=[0/1,0/1,0/1] Booleans to switch on/off terms
% thermal multimode scattering, shear multimode scattering, full dispersion
% relation
% Outputs:
% KEff(numDist,numVol,numFreq,numModels) effective wavenumber
% vel/att(numDist,numVol,numFreq,numModels) effective speed and attenuation
        
%Models
%1st order in concn: Foldy (1)
%2nd order in concn:
%    infinitely small scatterers (kb->0):
%           (2) no mode conversion mult scatt (LB)
%           (3)	mode conversion mult scatt low concn expn (LCVB22)
%           (4)	mode conversion mult scatt dispersion relation (LCVB22)
%           (5)	
%  	finite size scatterers
%           (6) no mode conversion mult scatt (Gower)
%           (7) mode conversion mult scatt low concn expn
%           (8) mode conversion mult scatt dispersion relation 
%

%Multiple scattering options 
% (passed in as inputs in the array multiModeOptions)
thermMultScatt=multiModeOptions(1); %set to 1 to include thermal multiple scattering terms
shearMultScatt=multiModeOptions(2); %set to 1 to include shear multiple scattering terms
fullDispReln=multiModeOptions(3); %Set this to zero to avoid using the determinant solution

numFreq=max(size(Kf)); %number of frequencies
Kf=reshape(Kf,[numFreq 1]); %ensure it is a column vector
freq=reshape(freq,[numFreq 1]);
numSubs=size(radiusType,1); %number of different types of material (substances)
numSizes=size(radiusType,2); %number of different sizes/size distributions
numVols=size(volFract,2); %number of different concentrations

B=zeros(numSubs,numFreq,numVols,numSizes,2);
KEff=zeros(numSizes,numVols,numFreq,numModels);
vel=zeros(numSizes,numVols,numFreq,numModels);
att=zeros(numSizes,numVols,numFreq,numModels);

for iSubs=2:numSubs %do each substance (iSubs=1 iscontinuous phase; this code can only deal with a single dispersed substance)
    radius=squeeze(radiusType(iSubs,:)); %take the array of sizes
    for iFreq=1:numFreq %each frequency value
        omega=2*pi*1.0e6*freq(iFreq); %freqs are in MHz
        K=Kf(iFreq);        %compressional wavenumber K at frequency
        L=Lf(iFreq);        %thermal wavenumber K_T at frequency
        S=Sf(iFreq);        %shear wavenumber K_S at frequency


        for iVol=1:numVols  %each volume fraction
            vol=squeeze(volFract(iSubs,iVol));
            for iSizes=1:numSizes
                Ka=K*radius(iSizes);
                La=L*radius(iSizes);
                Sa=S*radius(iSizes);
                %b is the radius of the excluded volume (>=2*radius of scatterer if hard spheres)
                Kb=K*2*exclFac*radius(iSizes);
                Lb=K*2*exclFac*radius(iSizes);
                Sb=S*2*exclFac*radius(iSizes);

                % TT is numSubs,numModes,numFreq,numSizes,numOrders)
                % All coefficients are stored in scaled form but the CC
                % coefficients are not scaled
                T0=TT(iSubs,1,iFreq,iSizes,1);
                T1=TT(iSubs,1,iFreq,iSizes,2);
                if nMax>0
                   T2=TT(iSubs,1,iFreq,iSizes,3);
                end

                fac=vol/((K*radius(iSizes))^3);

                % First calculate first order scattering contributions
                firstOrder=[];
                % Determine the first order scattering to order nMax
                firstOrder(1)=1-3*1i*fac*squeeze(TT(iSubs,1,iFreq,iSizes,...
                    1)).'; % this gets the right array structure for firstOrder
                for n=1:nMax %first order scattering is sum(2n+1)An*factors
                    firstOrder(1)=firstOrder(1)-3*1i*fac*(2*n+1)*...
                        squeeze(TT(iSubs,1,iFreq,iSizes,n+1)).'; %up to nMax
                end
                % ***** Foldy first order scattering
                iModels=1; % model number 1 is Foldy scattering to first order in concentration
                KEff(iSizes,iVol,iFreq,iModels)=sqrt(K*K*(firstOrder));
                vel(iSizes,iVol,iFreq,iModels)=omega/...
                    real(KEff(iSizes,iVol,iFreq,iModels));
                att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));

                if imag(KEff(iSizes,iVol,iFreq,iModels))<0
                    % Negative attenuation not allowed, take the other branch in the
                    % square root to obtain negative phase velocity and positive
                    % attenuation
                    KEff(iSizes,iVol,iFreq,iModels)=-sqrt(K*K*(firstOrder));
                    vel(iSizes,iVol,iFreq,iModels)=omega/...
                        real(KEff(iSizes,iVol,iFreq,iModels));
                    att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));
                end

                % Then calculate second order scattering contributions
                % First with single mode (no wave mode conversion)
                % with and without the volume exclusion term

                % ********* Single mode second order
                % single mode second order; do not include volume exclusion
                % terms (point scatterers, LB67)
                %Analytical form of second order multiple scattering is only used to order n=2
                % in second order i.e. use Lloyd-Berry point scatterer
                % but only up to max quadrupole order
                secondOrder01=-27*fac*fac*(T0.*T1+2*T1.*T1); %to first terms in T

                if nMax>1 %assume nMax>=1 always
                    %include T2 terms for both nMax
                    secondOrder(1)=secondOrder01-27*fac*fac*...
                        (10*T0.*T2/3+11*T1.*T2+230*T2.*T2/21);
                elseif nMax==1
                    secondOrder(1)=secondOrder01;
                else %nMax=0
                    secondOrder(1)=zeros(size(secondOrder01));
                end

                secondOrderModels(1:5)=secondOrder(1); %all models without volume exclusion (i.e. point scatterers)

                iModels=2; % model number 2 is single mode point scatterers
                KEff(iSizes,iVol,iFreq,iModels)=sqrt(K*K*(firstOrder+secondOrder));
                vel(iSizes,iVol,iFreq,iModels)=omega/...
                    real(KEff(iSizes,iVol,iFreq,iModels));
                att(iSizes,iVol,iFreq,iModels)=...
                    imag(KEff(iSizes,iVol,iFreq,iModels));
                if imag(KEff(iSizes,iVol,iFreq,iModels))<0
                    % Negative attenuation not allowed, take the other branch in the
                    % square root to obtain negative phase velocity and positive
                    % attenuation
                    KEff(iSizes,iVol,iFreq,iModels)=-sqrt(K*K*(firstOrder+secondOrder));
                    vel(iSizes,iVol,iFreq,iModels)=omega/...
                        real(KEff(iSizes,iVol,iFreq,iModels));
                    att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));
                end


                % *************
                % Single mode second order; include volume exclusion terms e.g. Linton-Martin 2006, Gower 2018
                DD=DmultScattmultSpecies(nMax,Kb);
                %DD is D (Gower 2018) = d (Linton Martin 2006) = -n_l() (Luppe Conoir Valier Brasier 2022)
                delta2LL=0;

                for n=0:nMax
                    for nu=0:nMax
                        for l=abs(n-nu):2:n+nu
                            delta2_nnul=-sqrt((2*l+1)*(2*nu+1)*(2*n+1)*4*pi)*DD(l+1)...
                                *squeeze(TT(iSubs,1,iFreq,iSizes,n+1))...
                                *squeeze(TT(iSubs,1,iFreq,iSizes,nu+1))...
                                *GauntCoeff(n,nu,l,0,0,0);
                            %DD is D (Gower 2018) = d (Linton Martin 2006) = -n_l() (Luppe Conoir Valier Brasier 2022)

                            delta2LL=delta2LL+delta2_nnul;
                        end
                    end
                end %end of loops n, nu, l for second order term
                delta2LL=delta2LL*(-9i*fac*fac*Kb/2);
                secondOrderModels(6:numModels)=delta2LL; %all models with volume exclusion
                secondOrder(1)=delta2LL;

                iModels=6; % model number 6 is single mode finite sized scatterers (LM06, GSPA2018)
                KEff(iSizes,iVol,iFreq,iModels)=sqrt(K*K*(firstOrder+secondOrder));
                vel(iSizes,iVol,iFreq,iModels)=omega/...
                    real(KEff(iSizes,iVol,iFreq,iModels));
                att(iSizes,iVol,iFreq,iModels)=...
                    imag(KEff(iSizes,iVol,iFreq,iModels));

                %************%End of single mode multiple scattering at second order in conc

                %************% Multiple mode multiple scattering at second order in conc
                extraTherm=0;
                extraShear=0;

                %***********% Thermal-compressional multi-mode multiple scattering at second order in concn
                % based on Luppe Conoir and Norris 2012 JASA 131(2) pp1113-1120
                % implemented by V Pinfield Dec 2012-July 2013
                if thermMultScatt
                    %use the stored scaled scattering coefficients
                    % (see definitions at top of function)
                    Tct0Sc=TT(iSubs,2,iFreq,iSizes,1); %compressional incident, thermal scatterered
                    Ttc0Sc=TTP(iSubs,1,iFreq,iSizes,1); %thermal incident, compressional scattered
                    Ttt0Sc=TTP(iSubs,2,iFreq,iSizes,1); %thermal incident, thermal scattered

                    facLb=thermMultFac(radius(iSizes),radius(iSizes),K,L);
                    extraTherm=-27*fac*fac*(-1i)*...
                        (Tct0Sc.*Ttc0Sc)*(K*K*K/(3*L*(K*K-L*L)))*facLb;
                    %***This is the correct one to use ****

                    %{
                    % A version that uses the unscaled scattering coefficients
                    % denoted Tct0 and Ttc0 for the mode converting terms
    
                    h0Lb=-1i*exp(1i*Lb)/Lb;
                    h0pLb=1i*(1+1i/Lb)*h0Lb;
                    j0Kb=sin(Kb)/Kb;
                    j0pKb=((1/tan(Kb))-1/Kb)*j0Kb;
                    j0La=sin(La)/La;
                    h0La=-1i*exp(1i*La)/La;
    
                    QFac=(Lb*j0Kb*h0pLb-Kb*j0pKb*h0Lb);
                    %extraTherm=-27*fac*fac*(-1i)*...
                    %    (Tct0.*Ttc0)*(K*K*K*Lb/(3*L*(K*K-L*L)))*QFac;
    
                    % End of the version that uses unscaled scattering
                    coefficients (T matrices)
                    %}

                end     %end of thermal multiple scattering block
                %***********% End of thermal-compressional multi-mode multiple scattering at second order in concn

                %***********% Shear-compressional multi-mode multiple scattering at second
                %***********% order in concentration

                if shearMultScatt %shear multiple scattering
                    %based on LCVB22 (Luppe Conoir Valier-Brasier 2022)
                    % taking leading order terms only (up to dipole, with
                    % option to include quadrupole)

                    j1Sa=sqrt(pi/2)*besselj(1+0.5,Sa)/sqrt(Sa);
                    h1Sa=sqrt(pi/2)*besselh(1+0.5,Sa)/sqrt(Sa);
                    j2Sa=sqrt(pi/2)*besselj(2+0.5,Sa)/sqrt(Sa);
                    h2Sa=sqrt(pi/2)*besselh(2+0.5,Sa)/sqrt(Sa);
                    %convert the coefficients back to UNSCALED values
                    %dipole n=1
                    T1CS=TT(iSubs,3,iFreq,iSizes,2)/h1Sa; %compressional incident, shear scatterered
                    T1SC=TTP(iSubs,1,iFreq,iSizes,2)*j1Sa; %shear incident, compressional scattered
                    T1SS=TTP(iSubs,3,iFreq,iSizes,2)*j1Sa/h1Sa;%shear incident, shear scattered
                    %quadrupole n=2
                    if nMax>0
                        T2CS=TT(iSubs,3,iFreq,iSizes,3)/h2Sa; % comp inc, shear scatt, n=2
                        T2SC=TTP(iSubs,1,iFreq,iSizes,3)*j2Sa; %shear inc, compr scatt, n=2
                        T2SS=TTP(iSubs,3,iFreq,iSizes,3)*j2Sa/h2Sa; %shear inc, shear scatt, n=2
                    end
                    %********** Low concentration expansion multimode
                    % comp-shear

                    % just n=1, nu=1 terms
                    % fac=concn/(kca)^3 (calculated earlier)
                    N0_S=multiModeFac(0,K,S,radius(iSizes),radius(iSizes));
                    N1_S=multiModeFac(1,K,S,radius(iSizes),radius(iSizes));
                    N2_S=multiModeFac(2,K,S,radius(iSizes),radius(iSizes));
                    delCSFac=3*T1CS*T1SC*N0_S; %this is the leading term
                    extraShear=-9*fac*fac*(K*K/(K*K-S*S))*1i*Kb*delCSFac;
                    
                    %{
                    % ***% Option to add the next most significant terms
                    % in the low concentration expansion
                    % For n=1 (for compr), nu=2 (for shear field)
                    delCSFac=delCSFac+(-3*T1CS*T1SC*N2_S);
                    % then terms higher than dipole but of same order in kca as the one above,
                    % but still smaller than the leading term.
                    %
                    delCSFac=delCSFac+3*N1_S*(T1CS*T2SC+3*T2CS*T1SC)+...
                        5*N0_S*T2CS*T2SC;
                    extraShear=-9*fac*fac*(K*K/(K*K-S*S))*1i*Kb*delCSFac;

                    %}
                    
                    %*******% Full dispersion relation comp-shear multimode
                    % Taking only the monopole and dipole terms
                    % This is the matrix that multiplies the array of coefficients (A0C A1C A1S)
                    % Find the effective wavenumber that leads to zero determinant
                    % for non-trivial solution for the coefficients

                    if fullDispReln
                        
                        % Take an initial estimate for the effective
                        % wavenumber using first order (in concentration)

                        B_init=sqrt(K*K*(firstOrder(1)+...
                            secondOrderModels(1)+extraShear)); % initial estimate for effective wavenumber

                        fac_C=fac; %=vol/(Ka^3)
                        fac_S=vol/(Sa^3);

                        % Set up the matrix, that multiplies the
                        % homogenised field coefficients
                        % in the order {A0C A1C A1S} for m=0
                        % (longitudinal effective field)
                        % Includes terms up to partial wave order 1 in both comp and shear
                        % using UNSCALED coefficients
                        % This is part of hte matrix that does not depend
                        % on the effective wavenumber
                        % It is incorporated into the full matrix using the
                        % function detDispReln

                        HH=[-T0 1i*T1 1i*T1SC; -3i*T0 -T1 -T1SC; 0 -T1CS -T1SS];

                        % Use the optimser to search for the effective
                        % wavenumber with the smallest determinant of the
                        % matrix

                        %options=optimset ('TolFun',1E-6,'TolX',1E-6,'Display','on','PlotFcn','optimplotfval');
                        %full plotting
                        options=optimset ('TolFun',1E-6,'TolX',1E-6,'OutputFcn',...
                            {@plotdet_ReB @plotdet_ImB @plotiter_ReB @plotiter_ImB});
                        %without plotting in optimiser
                        options=optimset ('TolFun',1E-6,'TolX',1E-6);

                        BSoln_ReIm=fminsearch(@(B_ReIm) detDispReln(B_ReIm,K,S,fac_C,fac_S,HH,radius(iSizes),...
                            radius(iSizes),exclFac),[real(B_init) imag(B_init)],options);
                        %*************
                        % This is the dispersion relation solution for the
                        % effective wavenumber
                        BSoln=complex(BSoln_ReIm(1),BSoln_ReIm(2));

                        % ******* Save result for dispersion relation
                        % Model number 8 is for finite sized scatterers,
                        % with mode conversion and no correlation
                        iModels=8;
                        KEff(iSizes,iVol,iFreq,iModels)=BSoln;
                        vel(iSizes,iVol,iFreq,iModels)=omega/...
                            real(KEff(iSizes,iVol,iFreq,iModels));
                        att(iSizes,iVol,iFreq,iModels)=...
                            imag(KEff(iSizes,iVol,iFreq,iModels));
                        if imag(KEff(iSizes,iVol,iFreq,iModels))<0
                            % Negative attenuation not allowed, take the other branch in the
                            % square root to obtain negative phase velocity and positive
                            % attenuation
                            KEff(iSizes,iVol,iFreq,iModels)=-sqrt(K*K*(firstOrder));
                            vel(iSizes,iVol,iFreq,iModels)=omega/...
                                real(KEff(iSizes,iVol,iFreq,iModels));
                            att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));
                        end


                        %{
                        % For display, check the initial and final
                        % estimates for effective wavenumber
                        det_MFnInit=detDispReln([real(B_init) imag(B_init)],K,S,fac_C,fac_S,HH,radius(iSizes),radius(iSizes),1);
                        det_MFnMin=detDispReln([real(BSoln_ReIm) imag(BSoln_ReIm)],K,S,fac_C,fac_S,HH,radius(iSizes),radius(iSizes),1);

                        [abs(det_MFnInit) abs(det_MFnMin)]; %display the starting and minimised determinants

                        %}
                    end %End of option to get the solution to the dispersion relation rather than the low concn expansion
                end %end of shear multiple scattering block

                secondOrder(1)=secondOrder(1)+extraTherm+extraShear;
                secondOrderModels(1:5)=secondOrderModels(1:5)+extraTherm+extraShear; %no vol exclusion in the CC term i.e. points
                secondOrderModels(6:numModels)=secondOrderModels(6:numModels)+extraTherm+extraShear; %with vol exclusion in the CC terms

                % Model number 3 is the point scatterer version for CC
                % terms, but then including mode conversion in the low
                % concentration expansion form
                iModels=3;
                KEff(iSizes,iVol,iFreq,iModels)=sqrt(K*K*(firstOrder+secondOrderModels(iModels)));
                vel(iSizes,iVol,iFreq,iModels)=omega/...
                    real(KEff(iSizes,iVol,iFreq,iModels));
                att(iSizes,iVol,iFreq,iModels)=...
                    imag(KEff(iSizes,iVol,iFreq,iModels));
                if imag(KEff(iSizes,iVol,iFreq,iModels))<0
                    % Negative attenuation not allowed, take the other branch in the
                    % square root to obtain negative phase velocity and positive
                    % attenuation
                    KEff(iSizes,iVol,iFreq,iModels)=-sqrt(K*K*...
                        (firstOrder+secondOrderModels(iModels)));
                    vel(iSizes,iVol,iFreq,iModels)=omega/...
                        real(KEff(iSizes,iVol,iFreq,iModels));
                    att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));
                end


                % Model number 7 is using finite sized scatterers for the CC terms
                % and then including mode conversion in the low
                % concentration expansion form
                iModels=7;
                KEff(iSizes,iVol,iFreq,iModels)=sqrt(K*K*...
                    (firstOrder+secondOrderModels(iModels)));
                vel(iSizes,iVol,iFreq,iModels)=omega/...
                    real(KEff(iSizes,iVol,iFreq,iModels));
                att(iSizes,iVol,iFreq,iModels)=...
                    imag(KEff(iSizes,iVol,iFreq,iModels));
                if imag(KEff(iSizes,iVol,iFreq,iModels))<0
                    % Negative attenuation not allowed, take the other branch in the
                    % square root to obtain negative phase velocity and positive
                    % attenuation
                    KEff(iSizes,iVol,iFreq,iModels)=-sqrt(K*K*...
                        (firstOrder+secondOrderModels(iModels)));
                    vel(iSizes,iVol,iFreq,iModels)=omega/...
                        real(KEff(iSizes,iVol,iFreq,iModels));
                    att(iSizes,iVol,iFreq,iModels)=imag(KEff(iSizes,iVol,iFreq,iModels));
                end


            end % end of loop for different sizes
        end % end of loop for different concentrations
    end % end of loop for different frequencies
end % end of loop for different substances (but this is only function for iSubs=2, single dispersed phase substance)
