function [KEff, velocity, attenuation] = multScattmultSpecies(nMax,radiusType,fractVolType,volFractTotType,freq,Kf,scattCoeff,exclFac,numModels)
%This function calculates the effective wavenumber using acoustic K,,
%multiple scattering, generalised to a multiple species system 
%(acoustic only, no mode conversion)
%where different species means both different sizes and types of particle
%A.L. Gower, M.J.A. Smith, W.J. Parnell, I.D. Abrahams, 
%Reflection from a multi-species material and its transmitted effective wavenumber, 
%Proc. R. Soc. A Math. Phys. Eng. Sci. 474 (2018) 20170864. 
%https://doi.org/10.1098/rspa.2017.0864.
%Supplementary materials 
% For a single scatterer species (monodisperse, single size, one dispersed
% phase substance), the result is identical to Linton/Martin 2006 for
% acoustic multiple scattering
%Use functions GauntCoeff and DmultScattmultSpecies and W3j
%Calculates for a set of volume fractions for each phase
%and for multiple ensembles of sizes i.e. numDist sets of different sizes 
%for each type of particle
%the i'th ensemble uses the i'th size distribution for each substance
%and for each distribution, a set of concentrations - j'th concentration for
%each substance.
%e.g. Two dispersed phase substances, sunflower oil, silica
% sizes and concentrations: 
% sunflower (1, 2)microns, (0.1,0.15) vol fract
% silica (0.2,0.4) microns, (0.05,0.08) vol fract
%This would calculate for the configurations:
% sunflower 1 micron, phi 0.1, silica 0.2 micron, phi 0.05
% sunflower 1 micron, phi 0.15, silica 0.2 micron phi 0.08
% sunflower 2 microns, phi 0.1, slica 0.4 microns, phi 0.05
% sunflower 2 microns, phi 0.15, slica 0.4 microns, phi 0.08
%and these would be calculated for every frequency
%
%Input:
%nMax: the maximum partial wave order required
%UNITS: radii in metres, frequency in MHz
%For monodisperse:
%radiusType(numSubs,numDist) and fractVolType(numSubs,numDist)
%For polydisperse:
%radiusType(numSubs,numDist,sizeBin) and fractVolType(numSubs,numDist,sizeBin)
%where numDist is the number of different monodisperse radii or the number of
%different distributions. Only work with the same i'th distribution or size
%for every substance
%For both types (mono/polydisperse)
%volFractTotType(numSubs,numVol) is the total volume fraction of particles the type
%iSubs. 
%K(freq): the wavenumber of the continuous phase for the compressional acoustic
%mode
%scattCoeff(numSubs,numModes,numFreq,numDist,numBins,nMax+2): the scattering coefficients
%exclFac is a numeric factor used on the exclusion radius
%exclFac*(rad1+rad2) for a pair of particles
%Output:
%KEff(numDist,numVol,numFreq,numModels)
% velocity(numDist,numVol,numFreq,numModels) and 
% attenuation(numDist,numVol,numFreq,numModels)
% Multiple scattering models are
% 1st order in concn: Foldy (1)
% 2nd order in concn:
%    infinitely small scatterers (kb->0):
%           (2) no mode conversion mult scatt (LB67, LM06, GSPA18)
%           (3)	mode conversion mult scatt low concn expn (LCVB22)
%           (4)	mode conversion mult scatt dispersion relation (LCVB22)
%           (5)
%  	finite size scatterers
%           (6) no mode conversion mult scatt (GSPA18)
%           (7) mode conversion mult scatt low concn expn
%           (8) mode conversion mult scatt dispersion relation
%

%Calculate for a set of numDist ensembles taking the i'th distribution for each type with the i'th distribution of all other types
%use only a single volume fraction set (i.e. a single value of vol fract
%for each type)

numModes=3; %compressional, thermal, shear modes
numFreq=max(size(Kf)); %number of frequencies
Kf=reshape(Kf,[numFreq 1]); %ensure it is a column vector
freq=reshape(freq,[numFreq 1]);
numSubs=size(radiusType,1); %number of different types of material (substances)
numDist=size(radiusType,2); %number of different sizes/size distributions
numVol=size(volFractTotType,2); %number of different concentrations
KEff=zeros(numDist,numVol,numFreq,numModels);
velocity=zeros(numDist,numVol,numFreq,numModels);
attenuation=zeros(numDist,numVol,numFreq,numModels);
%work out the size distributions: mono or polydisperse
nDimsRad=size(size(radiusType),2); 
if nDimsRad<3 %monodisperse, no size bin dimension in each size distribution
    numBins=1;
else %polydisperse 
    numBins=size(radiusType,3);%number of bins in each distribution
end

for iDist=1:numDist %work through each configuration (every i'th size distribution constructs a single ensemble)
    
    %*********************************************************************
    %Set up the collection of species and pairs
    
    %First construct the "species" list from the substances and sizes
    %so every different size OR type counts as a different species
    numSpecies=(numSubs-1)*numBins;%iSubs=1 is continuous phase
    iSpecies=0; %count species
    %disp(['frequency value number' num2str(iFreq)]);
    for iSubs=2:numSubs %each substance/type of particle
        for iBin=1:numBins %each size bin in the size distribution
            iSpecies=iSpecies+1;
            radiusSpecies(iSpecies)=radiusType(iSubs,iDist,iBin);
            fractVolSpecies(iSpecies)=fractVolType(iSubs,iDist,iBin);
            typeSpecies(iSpecies)=iSubs;
            sizeBinSpecies(iSpecies)=iBin;
            %            coeffSpecies(iSpecies,:,:,:)=scattCoeff(iSubs,:,:,iSize,:);%substance,mode,freq,size,order
        end
    end
    numSpecies=iSpecies; %the count of number of different species
    
    % Calculate the combined radius (excluded volume) and function D
    % for each combination of species in this
    % distribution/configuration
    iPair=1;
    for iSpecies=1:numSpecies
        for jSpecies=1:numSpecies
            %Gower - inter-centre distance of i,j combination
            combRadSpecies(iPair)=exclFac*...
                (radiusSpecies(iSpecies)+radiusSpecies(jSpecies));
            iPair=iPair+1;
        end
    end
    numPairs=iPair-1;
    
    %construct the Ka_ij values for all frequencies and pairs
    Kfaa=gmultiply(Kf,combRadSpecies); %K(f)*a_ij numFreq*numRad
    
    %*****************************************************************
    %Calculate the function Dm(x) for each order, freq,pair
    D_nfaa=DmultScattmultSpecies(nMax,Kfaa);
    %Calculate the limit of small radius to neglect excluded volume
    %and approach Lloyd/Berry result
    invKfaa=1./shiftdim(Kfaa,-1);% create array size 2nMax+1,numFreq,numPairs
    D0_nfaa=1i*gmultiply((0:2*nMax)',invKfaa); %the D function limit of small radius order,freq,pair
    
    %**************************************************************************
    % Calculate the multiple scattering terms by looping through each
    % species pair (i.e. combo of sizes or substances)
    firstOrder=0; %i.e. the KEff=K term
    secondOrder=0;
    secondOrderLB=0;
    %once the species are known, calculate the first and second order
    %scattering terms by combining scattering for each species
    
    iPair=0;
    for iSpecies=1:numSpecies
        %disp(['outer species value number' num2str(iSpecies)]);
        
        %************************************************************
        %First order summation
        %first order has to sum over all species and all orders
        %sum(Tnj*volj/(K*rj)^3) with volj=totalvolofSubs*fractVolofsizerj
        %scattCoeff(numSubs,numModes,numFreq,numDist,numBins,nMax+2)
        %first order term uses scatt coeffs as defined in ECAH - opposite
        %sign to the definition Zn in Linton Martin 2006
        iSubs=typeSpecies(iSpecies); %the particle type relating to this species
        iBin=sizeBinSpecies(iSpecies);%the size bin relating to this species
        volRadiSpecies=volFractTotType(iSubs,:)*fractVolSpecies(iSpecies)./...
            ((radiusSpecies(iSpecies)).^3); %phi/(r)^3 for this species
        %size=1*numVol
        for n=0:nMax
            %firstOrder=sum(3i(2n+1)(phi/(kr)^3)*(Tn)
            firstOrder=firstOrder-3i*(2*n+1)*...
                gmultiply(volRadiSpecies.',...
                (squeeze(scattCoeff(iSubs,1,:,iDist,iBin,n+1))./(Kf.^3)).');
        end
        
        %************************************************************
        %Second order summation
        %calculate the second order terms by summing over all pairs of
        %species (i.e. all size bins and all types of particle)
        %Keff^2=k^2*(1+firstOrder+secondOrder) so secondOrder=delta2*(n^2)/k^2
        %calculate the second order term in concentration: F00*(4pi*n)^2/k6 (eqn 10 of Gower supplementary)
        %but expressed as volume fractions .
        %Calculate with/without excluded volume (Gower; LloydLloyd/Berry/Linton/Martin)
        for jSpecies=1:numSpecies
            iPair=iPair+1;
            % disp(['inner species value number' num2str(jSpecies)]);
            jSubs=typeSpecies(jSpecies); %the particle type relating to this species (jSpecies)
            jBin=sizeBinSpecies(jSpecies);%the size bin relating to this species (jSpecies)
            %volRadjSpecies corrected 5/3/21 for jSubs not iSubs
            volRadjSpecies=volFractTotType(jSubs,:)*fractVolSpecies(jSpecies)./...
                ((radiusSpecies(jSpecies)).^3); %phi/(r)^3 for this species (jSpecies)
            %size=numVol*numFreq:
%            volRadjSpecies=gmultiply(volFractTotType(iSubs,:)',volRadjSpecies);
            sumn=0;
            sumnLB=0;
            for n=0:nMax
                sump=0;
                sumpLB=0;
                for p=0:nMax
                    sumq=0;
                    sumqLB=0;
                    for q=abs(n-p):2:n+p %valid values of q given by Gower 2018, after Eq A.2, according to rules for Gaunt coeffs
                        sumq=sumq+sqrt(2*q+1)*...
                            GauntCoeff(n,p,q,0,0,0).*squeeze(D_nfaa(q+1,:,iPair).');
                        sumqLB=sumqLB+sqrt(2*q+1)*...
                            GauntCoeff(n,p,q,0,0,0).*squeeze(D0_nfaa(q+1,:,iPair).');
                    end %end of order q loop
                    facpPair=sqrt(2*p+1)*squeeze(scattCoeff(jSubs,1,:,iDist,jBin,p+1)); %j'th species p'th order
                    sump=sump+sumq.*facpPair;
                    sumpLB=sumpLB+sumqLB.*facpPair;
                end % end of order p loop
                facnPair=sqrt(2*n+1)*squeeze(scattCoeff(iSubs,1,:,iDist,iBin,n+1));%i'th species, n'th order
                sumn=sumn+sump.*facnPair;
                sumnLB=sumnLB+sumpLB.*facnPair;
            end %end of order n-loop
            facPair=combRadSpecies(iPair)...
                .*volRadiSpecies.*volRadjSpecies; %numVol*1
            secondOrder=secondOrder+gmultiply(facPair.',(sumn./(Kf.^5)).'); %numVol*1  * 1*numFreq->numVol*numFreq
            secondOrderLB=secondOrderLB+gmultiply(facPair.',(sumnLB./(Kf.^5)).');
            
        end %end of j'th Species
    end %end of i'th Species
    %secondOrder(2)=secondOrder(1); %this should be including an extra order nMax+1 but too complicated.
    secondOrder=secondOrder*9i*sqrt(pi);%delta2*n^2/k^2 (Keff^2=k2(1+firstOrd+secondOrd))
    secondOrderLB=secondOrderLB*9i*sqrt(pi);%delta2*n^2/k^2 (Keff^2=k2(1+firstOrd+secondOrd))
    
    %*************************************************************************
    %Store all the alternative multiple scattering models
    %reversing the sign for all values of effective wavenumber which have
    %negatiive imaginary part. This results in negative speed and positive
    %attenuation as required by energy considerations.
    %Single scattering (Foldy) - model number 1
    Kf_vol=repmat(Kf.',[numVol 1]); %duplicate the Kf array into the right shape numVol*numFreq
    %KEff(numDist,numVol,numFreq,numModels)
    firstOrdSqrt=sqrt(Kf_vol.*Kf_vol.*(1+firstOrder));
    firstOrdNeg=imag(firstOrdSqrt)<0; %identify where negative attenuation occurs
    KEff(iDist,:,:,1)=firstOrdSqrt.*~firstOrdNeg-firstOrdSqrt.*firstOrdNeg;
    %Lloyd Berry multiple acoustic scattering (no excluded volume)
    %Gower/LM with infinitesimally small scatterers
    %identical to Povey polydisperse method for L&B
    %model number 2
    LBSqrt=sqrt(Kf_vol.*Kf_vol.*(1+firstOrder+secondOrderLB));
    LBNeg=imag(LBSqrt)<0; %identify where negative attenuation occurs
    KEff(iDist,:,:,2)=LBSqrt.*(~LBNeg)-LBSqrt.*LBNeg;
    
    %Multiple acoustic scattering with excluded volume (Gower) - model number 6
    fullSqrt=sqrt(Kf_vol.*Kf_vol.*(1+firstOrder+secondOrder));
    fullNeg=imag(fullSqrt)<0; %identify where negative attenuation occurs
    KEff(iDist,:,:,6)=fullSqrt.*(~fullNeg)-fullSqrt.*(fullNeg); %combining gives the positive values unchanged, but the values with negative imaginary parts change sign
    
    freq_vol=repmat(freq.',[numVol 1]); %reshape to numVol*numFreq
    for iModel=1:numModels
        velocity(iDist,:,:,iModel)=2*pi*1.0e6*freq_vol./shiftdim(real(KEff(iDist,:,:,iModel)),1);
        attenuation(iDist,:,:,iModel)=shiftdim(imag(KEff(iDist,:,:,iModel)),1);
    end
end %end of loop for different ensembles (distributions)

