function [scattCoeffn,scattCoeffPn,matrixNearSing,nMax_OK]=...
    scattCoeffAsympt...
    (nMax,propsCont,propsDisp,f,a,jn0,hn,jn,K,L,S,Kp,Lp,Sp,...
    gammaC,gammaT,gammaCp,gammaTp,tempK,thermOption)

%Asymptotic solutions by Valerie Pinfield for single subst, freq and size
%calculate the scattering coefficients An,Bn,Cn etc including multi-mode terms
% Often called T matrix, with TnCC and mode converting terms
%
%Inputs:
%nMax:maximum order n, calculates to nMax+1 and array index is (n+1)
%propsCont and propsDisp are structures for properties of continuous and
%dispersed phases,with (at least) fields 'density',
%'shearViscMod','conduct','matType'
%f:freq in MHz, a: radius in metres,
%jn0(iDeriv+1,n+1): incident field jn(Ka) for derivative iDeriv and order n
%hn(iMode,iDeriv+1,n+1) for cont phase field modes
%jn(iMode,iDeriv+1,n+1) for disp phase field modes
%K,L,S: wavenumbers of propl, thermal, shear modes in cont phase
%Kp,Lp,Sp: wavenumbers of propl, thermal, shear modes in disp phase
%gammaC,T,S: thermal factors for propl, thermal, shear modes, contin phase
%gammaCp,Tp,Sp: similarly for disp phase
% tempK temperature in K
%thermOption: 1/0 option to include/exclude thermal boundary equations
%
%Outputs:
% An,Bn,Cn=scattCoeffn(mode 1:3,n+1) are scattering coefficients of the
% fields of different modes, with a compressional incident wave in order:
% compressional, thermal, shear
% scattCoeffPn(mode 1:3,n+1) are scattering coefficients of the each mode 
% for different incident modes: thermal incident for n=0,shear incident for
% n>=1. So for n=0 they are the T0TC and T0TT in (i=1,i=2)
% and for n=1 they are T1SC and T1SS in (i=1 and i=3) etc.
%coefficients for propl, thermal and shear modes in contin and disp phases
%matrixNearSing is a flag='true' if any matrix was nearly singular

persistent alreadyAskednMax %flag whether user has confirmed nMax too small but continuing
if isempty(alreadyAskednMax)
    alreadyAskednMax=false; %initalise it on the first function call
end
nMax_OK=true;
matrixNearSing=false; %set false flag for this call to function

scattCoeffn=zeros(3,nMax+2);
scattCoeffPn=zeros(3,nMax+2);

iOmega=1i*2e6*pi*f; %f is in MHz
pressFac=iOmega*propsCont.density;
%equations are written as for liquids
%for solids, convert viscosity and multiply by i*omega
%dispersed phase type
if strcmpi(propsDisp.matType,'solid')
    isCavity=0;
    etaP=propsDisp.shearViscMod/(-iOmega);
elseif strcmpi(propsDisp.matType,'cavity')
    isCavity=1;
else %liquid
    isCavity=0;
    etaP=propsDisp.shearViscMod;
end
%continuous phase type
if strcmpi(propsCont.matType,'solid')
    etaC=propsCont.shearViscMod/(-iOmega);
else %liquid
    etaC=propsCont.shearViscMod;
end

if ~isCavity
    etaHat=etaP/etaC;
    torHat=propsDisp.conduct/propsCont.conduct;
    rhoHat=propsDisp.density/propsCont.density;
end
%ignore bulk viscosity: it is included through attenuation.
% But ignore the equivalent for a solid

Ka=K*a;
La=L*a;
Sa=S*a;
Kpa=Kp*a;
Lpa=Lp*a;
Spa=Sp*a;
LHS=zeros(6,6);
RHS=zeros(6,1);
coeffs=zeros(1,6);

%************Check the number of orders needed
if(abs(Ka)>0.5)
    nReqd=ceil(abs(Ka))+5; %Ka+5 is the number of orders needed
elseif (abs(Ka)<0.05)
    nReqd=2; %only orders up to 2 if in long wavelength limit. Solids need n=2 too
    nReqd=nMax; %to use the specified number of orders
else
    nReqd=nMax; %otherwise do as requested
end

if (nReqd>(nMax+1))
    nLast=nMax+1;  %only up to full array size
    if ~alreadyAskednMax %if haven't warned the user yet, ask if continue or abort
        alreadyAskednMax=true; %flag up that the user has been warned
        [OK_Cancel]=questdlg(sprintf('%40s\n%40s','nMax is not big enough. Need nMax>Ka+5:',...
            'Click OK to continue or Cancel to abort calculation'),...
            'Inaccurate calculation','OK','Cancel','OK');
        if strcmp(OK_Cancel,'Cancel')
            nMax_OK=false;
            return
        end
    end
else
    nLast=nReqd; %usual calculation
end

% These are the solutions when the scatterer is not a cavity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Cycle through orders n

% **** n=0 solution first
if thermOption %include thermal effects
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %suspension: fluid continuous, solid dispersed phase
    c=propsCont.soundVel;
    cl2=c*c;  %to avoid imaginary part to the velocity in liquids
    bulkMod=cl2*propsCont.density;
    ratioCpCv_1=tempK*propsCont.expans*propsCont.expans*...
        bulkMod/(propsCont.density*propsCont.heatCapCp);
    H=1/((1/(1-1i*La))-propsCont.conduct*tan(Lpa)/...
        (propsDisp.conduct*(tan(Lpa)-Lpa)));
    T0CC=-(1i*Ka/3)*(Ka*Ka-Kpa*Kpa/rhoHat)-1i*(Ka^3)*(ratioCpCv_1)*H*...
        ((1-propsDisp.expans*propsCont.heatCapCp*propsCont.density/...
        (propsCont.expans*propsDisp.heatCapCp*propsDisp.density))^2)/...
        (La*La);

    %************* Now the mode-conversion coefficients for n=0
    % T0CT: scaled by h0(La): stored as T0CT*h0(La)
    %analytical solution for the thermal scattered wave coefficient
    %with compressional incident wave
    %the analytical result is for scaled thermal scattered waves by h0(kTa)=-i*exp(i*kTa)/kTa
    %=T0CT*h0(La)
    %calculate the Bessel functions for the incident thermal wave
    j0th=sin(La)/La;
    j0pth=(cos(La)-j0th)/La;

    b2=(gammaT/gammaC)*(1-La*hn(2,2,1)*jn(2,1,1)/...
        (torHat*Lpa*hn(2,1,1)*jn(2,2,1)));
    T0CTSc=-(1-gammaCp/(gammaC*rhoHat))/b2;
    %{
        %low frequency limit (small La) scaled analytical result for T0CT
        gammaCpCv_1=propsCont.soundVel*propsCont.soundVel*propsCont.expans*...
            propsCont.expans*298.15/propsCont.heatCapCp;
        T0CTScLow=(torHat/3)*(Lpa*Lpa)*(gammaC/gammaT)*...
            (1-gammaCp/(rhoHat*gammaC))*(1+1i*La-La*La+...
            Lpa*Lpa*(1+5*torHat)/15-...
            1i*La*La*La+1i*La*Lpa*Lpa*(1+10*torHat)/15);
        %low frequency limit (small La) unscaled analytical result
        T0CTUnLow=1i*(torHat/3)*(La*Lpa*Lpa)*(gammaC/gammaT)*...
            (1-gammaCp/(rhoHat*gammaC))*(1-La*La/2+...
            Lpa*Lpa*(1+5*torHat)/15-...
            1i*La*La*La/3+1i*La*Lpa*Lpa*torHat/3);
        %End of low frequency (small La) limit for T0CT
    %}

    %*******************************
    % T0TC: scaled by j0(La): stored (scaled) as T0TC/j0(La)
    %analytical solution for thermal incident wave, compressional
    %scattered wave

    j0pTpS=jn(2,2,1)/jn(2,1,1);
    h0pTS=hn(2,2,1)/hn(2,1,1);
    T0TCSc=1i*Ka*La*(j0pth/j0th)*(1-gammaT/(torHat*gammaTp))*...
        (1-h0pTS*j0th/j0pth)/(1-(La/(Lpa*torHat))*h0pTS/j0pTpS);
    %{
        %low frequency limit analytical solution (small La): scaled
        %coefficient T0TCSc
        T0TCScLow=(-1i*Ka*Lpa*Lpa*torHat/3)*(1-gammaT/(torHat*gammaTp))*...
            (1-La*La/3+(Lpa*Lpa/15)*(1+5*torHat)...
            -1i*La*La*La/3+1i*torHat*La*Lpa*Lpa/3);
        %low freq limit analytical solution (small La): unscaled
        %coefficient T0TC
        T0TCUnLow=(-1i*Ka*Lpa*Lpa*torHat/3)*(1-gammaT/(torHat*gammaTp))*...
            (1-La*La/2+(Lpa*Lpa/15)*(1+5*torHat)...
            -1i*La*La*La/3+1i*torHat*La*Lpa*Lpa/3);
        % End of low frequency limit for T0TC
    %}

    % *********************************************
    % T0TT: scaled by h0(La) and j0(La): stored (scaled as
    % T0TT*h0(La)/j0(La))
    % analytical solution for thermal incident wave, thermal scattered wave
    T0TTSc=-((j0pth/j0th)*La-Lpa*torHat*j0pTpS)/(La*h0pTS-Lpa*torHat*j0pTpS);
    T0TT=j0th*T0TTSc/hn(2,1,1);

    %   stored coefficient options: storing scaled coefficients
    scattCoeffn(1,1)=T0CC;
    scattCoeffn(2,1)=T0CTSc; % T0CT*h0(La)
    scattCoeffPn(1,1)=T0TCSc; % T0TC/j0(La)
    scattCoeffPn(2,1)=T0TTSc; % T0TT*h0(La)/j0(La)

    %*********************
else %non-thermal solution for n=0
    T0CC=-(1i*Ka/3)*(Ka*Ka-Kpa*Kpa/rhoHat);
    scattCoeffn(1,1)=T0CC; %mode 1 is compressional
end % end of thermal or non-thermal option for n=0

% **********************************
% n=1 incident shear wave calculation
n=1;
%Incident shear wave
%calculate the Besselj functions for the incident shear wave
j0sh=sin(Sa)/Sa;
j1sh=(1/Sa)*(sin(Sa)/Sa-cos(Sa));
j1psh=j0sh-2*j1sh/Sa;
j1dpsh=((6-Sa*Sa)*j1sh-2*Sa*j0sh)/(Sa*Sa);
j2sh=(1/Sa)*((3/(Sa*Sa)-1)*sin(Sa)-(3/Sa)*cos(Sa));

%Analytical Solutions for the shear wave coefficients - not scaled
denom=3*hn(3,1,3)-2*(rhoHat-1)*hn(3,1,1);
T1CC=1i*(Ka*Ka*Ka/3)*(rhoHat-1)*hn(3,1,3)/denom;
T1CS=-(Ka/Sa)*(rhoHat-1)/denom;
T1SC=-(2i*Sa*Ka*Ka/3)*(rhoHat-1)*(hn(3,1,3)*j0sh-...
    hn(3,1,1)*j2sh)/denom;
T1SS=-(3*j2sh-2*(rhoHat-1)*j0sh)/denom;

%Save the analytical coefficients (scaled)

scattCoeffn(1,n+1)=T1CC;
scattCoeffn(3,n+1)=T1CS*hn(3,1,2); %mode 3 (Cn) for n SCALED by h1(Sa)
scattCoeffPn(1,n+1)=T1SC/j1sh; %T1SC scaled by /j1(Ksa)
scattCoeffPn(3,n+1)=T1SS*hn(3,1,2)/j1sh; %T1SS *h1(ksa)/j1(ksa)

% All other orders, all analytical solutions are without thermal effects
%This section of code is not fully tested
for n=2:nLast %each order
    jnSa=sqrt(pi/2)*besselj(n+0.5,Sa)/sqrt(Sa);
    hnSa=hn(3,1,n+1);
    jn_1Sa=sqrt(pi/2)*besselj(n-0.5,Sa)/sqrt(Sa); %to calc the derivs
    jnpSa=(jn_1Sa-(n+1)*jnSa/Sa); %jnp=jn_1-(n+1)*jn/x

    denom=(n+1)*(Sa*hn(3,2,n+1)+(n+1)*hn(3,1,n+1));
    TnCC=(jn(1,1,n+1)/hn(1,1,n+1))*n*(Sa*hn(3,2,n+1)-n*hn(3,1,n+1))/...
        denom;
    TnCS=-jn(1,1,n+1)*(2*n+1)/denom;
    TnSC=(1/hn(1,1,n+1))*1i*n/denom;
    TnSS=-(Sa*jnpSa+(n+1)*jnSa)/denom;

    scattCoeffn(1,n+1)=TnCC;
    scattCoeffn(3,n+1)=TnCS*hnSa; %mode 3 (Cn) for n SCALED by hn(Sa)
    scattCoeffPn(1,n+1)=TnSC/jnSa; %TnSC scaled by /jn(Ksa)
    scattCoeffPn(3,n+1)=TnSS*hnSa/jnSa; %TnSS *hn(ksa)/jn(ksa)

end    % end of for n=2:nLast block

end %end of function

