% Script to conduct scattering calculations for the 3D case.
% using subfunctions to set up the configuration of the system, calculate
% the scattering coefficients, and then calculate the effective properties
% for the scattering system
% This version has 4 options for the scattering coefficients 
% and 2 options for the multiple scattering calculations 
% Multi-mode multiple scattering is based on Luppe Conoir Valier Braiser 2022
% with asymptotic analysis presented in Pinfield and Valier-Brasier, 
% Proceedings of the Royal Society 2024
% Multimode multiple scattering is only available for monodisperse, single
% scatterer species systems
%
%This script file is used for defining a system and
% executing the relevant calculations
% Key array sizes:
% KEff(numDist,numVol,numFreq,numModels),vel/att(numDist,numVol,numFreq,numModels)
% radiusType(numSubs,numDist,numBins); %set of all radii for each subs, distribution, in metres
% fractVol(numSubs,numDist,numBins); %set of fractional volume for each subs, distrib
% K(numSubs,numModes,numFreq)

%***********************User settings *************************************
mono=1; %monodisperse=1, polydisperse=0
coeffOption=1; % Options for scattering coefficients: 
% 1=ECAH numerical, 2=AC core-shell operated as ECAH 3=acoustic only 4=asymptotic analytics
% Note that ECAH numerical solution can be edited to provide analytical
% versions of the coefficients
thermOption=0; % 1 or 0 to include/exclude thermal effects in boundary conditions for scattering coefficients
numSubs=2; %numSubs includes continuous phase
numMedRad=1; %number of different median radii for each dispersed substance
numSigmaR=1; %number of different widths to generate distributions for each substance if polydisperse
if mono numSigmaR=1; end % must be 1 for monodisperse
numVol=1; %number of different volume fractions for each substance
numFreq=991;
numModes=3; %wave modes: compressional, thermal, shear
numDist=numMedRad*numSigmaR;
medRad=zeros(numSubs,numMedRad); %the "median" radii in micrometres
sigmaR=zeros(numSubs,numSigmaR); %width of size distribution in micrometres
volFractTot=zeros(numSubs,numVol);
freq=zeros(numFreq,1);
nMax=1;
exclFac=1.0; %the excluded radius factor (radius of excluded volume = exclFac*(radius_i+radius_j)
%ALL RADII ARE INITIALLY STORED IN MICROMETRES AND FREQ IN MHZ
%AND THEN RADII ARE CONVERTED TO METRES FOR SCATT COEFF CALCNS/MULT SCATT
%monodisperse, single or multiple  dispersed phase
distType='lognormal'; %the model for the size distribution for polydisperse - needs a function called by this name
if mono
    numBins=1;
    medRad(2,:)=[0.5]; %micron medRad for subs 2: 1*numMedRad these are the monodisperse radii
    % multiModeOptions=[0/1,0/1,0/1] Booleans to switch on/off terms
    % thermal multimode scattering (on/off), shear multimode scattering (on/off)
    % full dispersion relation(1) or low concentration expansion(0)
    multiMode=true; %set this to true to use the multiple mode multiple scattering models (can switch them off through the multiModeOptions)
    multiModeOptions=[0 1 0];
else
    %polydisperse
    numBins=9;
    medRad(2,:)=[20.0]; %1*numMedRad these are the "centre" radii of the distributions
    %medRad(3,:)=[3];
    sigmaR(2,:)=[0.2]; %1*numSigmaR
    %sigmaR(3,:)=[0.2 0.25];
    distType='lognormal'; %the model for the size distribution for polydisperse - needs a function called by this name
    % multiMode: to switch on/off the multi-mode contributions to multiple scattering
    % multiModeOptions=[0/1,0/1,0/1] Booleans to switch on/off terms
    % thermal multimode scattering (on/off), shear multimode scattering (on/off)
    % full dispersion relation(1) or low concentration expansion(0)
    multiMode=false; %Always set to false for polydisperse; no multi mode multiple scattering calculations for polydisperse systems
    multiModeOptions=[0 0 0];
end
%if didn't include thermal terms in scattering coefficients, don't do thermal multi-mode scattering
if ~thermOption 
    multiModeOptions(1)=0; % no thermal multimode scattering when thermOption=0
end
volFractTot(2:numSubs,:)=repmat(linspace(0.3,0.3,numVol),[numSubs-1 1]); %volume fractions for each substance
freq=linspace(1,100,numFreq);%frequencies in MHz

%physical properties of substances****************************************
%(see VJP JASA paper 2017)
tempK=298.0; %temperature in Kelvin
physProps=[];
%water
physProps{1}.matType='liquid';
physProps{1}.soundVel=1497.0; %m/s
physProps{1}.density=997.0; %kg/m3
physProps{1}.shearViscMod=0.000891; %Pa s
physProps{1}.conduct=0.595; %J/m s K (thermal conductivity)
physProps{1}.heatCapCp=4179.0; %J/kg K
physProps{1}.expans=0.00021; %/K
physProps{1}.alphaFac=0.023;
physProps{1}.alphaExp=2.0;
%silica
%physProps{2}.subsName='silica';
physProps{2}.matType='solid';
physProps{2}.soundVel=5968.0; %5968m/s
physProps{2}.density=2041.0; %kg/m3 %experimentally determined density
physProps{2}.shearViscMod=30.9E9; %30.9E9 Pa
physProps{2}.conduct=1.6; %J/m s K (thermal conductivity)
physProps{2}.heatCapCp=729.0; %J/kg K
physProps{2}.expans=1.35E-6; %/K
physProps{2}.alphaFac=2.6E-10;
physProps{2}.alphaExp=2.0;
%{
%Polystyrene properties from Challis et al. 2005
iSubs=3;
%physProps{2}.subsName='polystyrene';
physProps{iSubs}.matType='solid';
physProps{iSubs}.soundVel=2330.0; %m/s
physProps{iSubs}.density=1053.0; %kg/m3
physProps{iSubs}.shearViscMod=1.27E9; %Pa
physProps{iSubs}.conduct=0.14; %J/m s K (thermal conductivity)
physProps{iSubs}.heatCapCp=1193.0; %J/kg K
physProps{iSubs}.expans=0.00021; %/K
physProps{iSubs}.alphaFac=0.1;%Np/m/MHz^alphaExp
physProps{iSubs}.alphaExp=2.0;
%}

%{
%Properties for sunflower oil in water
%(see VJP PhD thesis, page 175)
tempK=303.15; %temperature in Kelvin
physProps=[];
%water
physProps{1}.matType='liquid';
physProps{1}.soundVel=1509.127; %m/s
physProps{1}.density=995.65; %kg/m3
physProps{1}.shearViscMod=0.0007973; %Pa s
physProps{1}.conduct=0.6032; %J/m s K (thermal conductivity)
physProps{1}.heatCapCp=4178.2; %J/kg K
physProps{1}.expans=0.000301; %/K
physProps{1}.alphaFac=0.022;
physProps{1}.alphaExp=2.0;
%sunflower oil 
%physProps{2}.subsName='sunflower oil';
physProps{2}.matType='liquid';
physProps{2}.soundVel=1437.86; %m/s
physProps{2}.density=912.9; %kg/m3
physProps{2}.shearViscMod=0.054; %Pa s
physProps{2}.conduct=0.17; %J/m s K (thermal conductivity)
physProps{2}.heatCapCp=1980.0; %J/kg K
physProps{2}.expans=0.0007266; %/K
physProps{2}.alphaFac=1.0;
physProps{2}.alphaExp=1.7;
%}
if size(physProps,2)~=numSubs
    warndlg('physical properties array does not have the right number of substances')
end
%***********************End of user settings*******************************

%************** Run all scattering calculations ***************************
[KEff,vel,att,Tn,K,gammaCT]=...
scatterCalc3D(nMax,coeffOption,thermOption,multiMode,multiModeOptions,tempK,physProps,freq,...
    medRad,sigmaR,numBins,distType,volFractTot,exclFac);
%KEff(numDist,numVol,numFreq,numModels),vel/att(numDist,numVol,numFreq,numModels)

%*************************************************************************

%*************************************************************************
%plot results 
%velocity/atten(numDist,numVol,numFreq,numModels)
%Marker options for plotting
markers=['o','+','*','x','-','|','^','v',];
legendList={'Foldy', 'NoWC points','WC LC points','WC DR points',...
    '','NoWC LC vol excl','WC LC vol excl','WC DR vol excl'};

% Multiple scattering models are
%1st order in concn: Foldy (1)
%2nd order in concn:
%    infinitely small scatterers (kb->0):
%           (2) no mode conversion mult scatt (LB)
%           (3)	mode conversion mult scatt low concn expn (LCVB22)
%           (4)	mode conversion mult scatt dispersion relation (LCVB22)
%           (not assigned)
%           (5)	
%  	finite size scatterers
%           (6) no mode conversion mult scatt (Gower)
%           (7) mode conversion mult scatt low concn expn
%           (8) mode conversion mult scatt dispersion relation 
%
plotModels=[2 3 6 7]; %set which models to plot
iVol=numVol;
iDist=1;

% *****************************
% Plot speed against frequency
figure
hold all
for iModel=plotModels
    plot(freq,squeeze(vel(iDist,iVol,:,iModel)),markers(iModel))
end
legend(legendList(plotModels))
title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('freq / MHz')
ylabel('speed / m/s')

% *****************************
% Plot attenuation against frequency
figure
hold all
for iModel=plotModels
    plot(freq,squeeze(att(iDist,iVol,:,iModel)),markers(iModel))
end
legend(legendList(plotModels))
title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('freq / MHz')
ylabel('atten / Np/m')

% *****************************
% Plot real part of effective wavenumber against frequency
figure
hold all
for iModel=plotModels
    plot(freq,squeeze(real(KEff(iDist,iVol,:,iModel))),markers(iModel))
end
legend(legendList(plotModels))
title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('freq / MHz')
ylabel('real(KEff) / /m')

% *****************************
% Plot imaginary part of effective wavenumber against frequency
figure
hold all
for iModel=plotModels
    plot(freq,squeeze(imag(KEff(iDist,iVol,:,iModel))),markers(iModel))
end
legend(legendList(plotModels))

title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('freq / MHz')
ylabel('imag(KEff) / /m')


ksa=medRad(2,iDist)*1E-6.*K(1,3,:);
kca=squeeze(K(1,1,:))*medRad(2,iDist)*1E-6;
%Plot frequency spectrum against ksa only up to the limit kca=0.1
kcaLim=kca>0.1;
lastF=find(kcaLim,1);
lastF=numFreq; %plot all the data even outside the long wavelength instead

% *****************************
% Plot attenuation per wavelength against real(ksa)
figure
for iModel=plotModels
    semilogx(squeeze(real(ksa(1:lastF))),1E-6*squeeze(...
        att(iDist,iVol,1:lastF,iModel))*physProps{1}.soundVel./freq(1:lastF).'...
        ,markers(iModel))
    hold all
end
legend(legendList(plotModels))
title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('real(ksa)')
ylabel('atten per wavelength / Np')

% *****************************
% Plot speed against real(ksa)
figure
for iModel=plotModels
    semilogx(squeeze(real(ksa(1:lastF))),squeeze(vel(iDist,iVol,1:lastF,iModel)),...
        markers(iModel))
    hold all
end
legend(legendList(plotModels))
title(['radius ' num2str(medRad(2,iDist)) ' micr. Vol fract ' num2str(volFractTot(2,iVol))])
xlabel('real(ksa)')
ylabel('speed / m/s')

%}