function [radiusType,fractVol] = setUpSysParams(medianR,sigmaR,numBins,distType)
%Code to set up system parameters for a scattering system
%in an appropriate form to use the other functions e.g. for scattering
%coefficients and multiple scattering formula. 
%This is for both 2D and 3D cases
%This is to replace the GUI dialog boxes for setting up the scattering system in
%the most general possible way i.e. multiple substances, multiple size
%distributions and multiple volume fractions and frequencies
%Inputs: 
%ALL RADII IN MICRONS
%medianR(numSubs,numMedR)- the monodisperse radii or median radii for
%polydisperse for each substance
%sigmaR(numSubs,numSigmaR) - the widths of the polydisperse distributions
%such that the number of different distributions (mono or
%poly) numDist=numMedR*numSigmaR
%numBins - the numbers of size bins in each distribution (=1 for mono)
%NOTE: numSubs includes the continuous phase as the first subs
%distType: a string name of the model for size distribution e.g.
%"lognormal" - this must be the name of a function which calcultes that
%distribution
%Outputs
%radiusType(numSubs,numDist,numBins) - the radii of each size or size bins
%for each substance in microns
%fractVol(numSubs,numDist,numBins) fractional volumes (sum to unity for
%each distribution)

numSubs=size(medianR,1); %number of substances including continuous phase
numMedR=size(medianR,2); %number of different median radii
numSigma=size(sigmaR,2);%number of different distribution widths
numDist=numMedR*numSigma;
radiusType=zeros(numSubs,numDist,numBins);
fractVol=zeros(numSubs,numDist,numBins);

if numBins==1
    %monodisperse distribution
    radiusType(:,:,1)=medianR;
    fractVol(:,:,1)=1;
else
    %polydisperse distribution
    %generate the polydisperse distributions using the requested model
    %'distType'
    for iSubs=2:numSubs
        %works better in micrometres
        [fractr,fractv,delLnr,muSigma]=...
            feval(distType,squeeze(medianR(iSubs,:)),...
            squeeze(sigmaR(iSubs,:)),3,numBins);
        for iDist=1:numDist
            %normalise the distribution
            fractv(iDist,:)=fractv(iDist,:).*fractr(iDist,:)*delLnr(iDist); %fractv is normalised if bin width small enough
            fractv(iDist,:)=fractv(iDist,:)/sum(fractv(iDist,:)); %normalise it to the discretised bin fractions
        end
        radiusType(iSubs,:,:)=fractr;%calculated using micrometres, but store in metres
        fractVol(iSubs,:,:)=fractv;
    end
end
end

