function [scattCoeff]=sphAcScattCoeff(nMax,K,rho,Kp,rhop,cSpd,cpSpd,radius)
%The scattering coefficients for orders 0-nMax for a sphere
%using acoustics only (no shear waves or thermal waves)
%see Gower 2018, Eq 12 supplementary 
%for a single dispersed phase species
%k may be an array for different frequencies (K in /METRE)
%radius may be an array of different radii (RADII IN METRES)
%Outputs:
%scattCoeff(numFreq,numRad,nMax+1)
%NOTE these coefficients are the negative of the Zn coefficient stated by 
%Linton/Martin 2006 (and Gower 2018 multi-species)
%So here it produces a coefficient consistent with ECAH definitions
%LM2006 and Gower use a definition that includes the direction of
%propagation of the inicident wave, giving a factor of -1 (see LM Eq 4.5 and
%preceding definitions -the factor Anm is -1).

numFreq=max(size(K)); %number of frequencies
numRad=max(size(radius)); %number of radii
K=reshape(K,[numFreq 1]); %ensure it is a column vector
Kp=reshape(Kp,[numFreq 1]);
radius=reshape(radius,[1 numRad]);%ensure it is  a row vector

%shape the speeds for generalised frequency-dependence
if length(cSpd)==1
    cSpd=repmat(cSpd,[numFreq 1]);
end
if length(cpSpd)==1
    cpSpd=repmat(cpSpd,[numFreq 1]);
end

scattCoeff=zeros(numFreq,numRad,nMax+2);
%q is usually independent of frequency, but retain the possible freq-dep
q=repmat(cpSpd*rhop./(rho*cSpd),[1 numRad]); %numFreq*1 repmat ->numRad*numFreq
Ka=gmultiply(K,radius); %numFreq*numRad
Kpa=gmultiply(Kp,radius);
spherFactor=sqrt(pi/2); % to convert Jn to spherical bessel function jn
rootKa=sqrt(Ka);
rootKpa=sqrt(Kpa);

%for n=0 
j0_ka=spherFactor*besselj(0.5,Ka)./rootKa;
h0_ka=spherFactor*besselh(0.5,1,Ka)./rootKa;
%calculate for n=0 first using the recurrence relation with Z(n+1)
j0p_ka=-spherFactor*besselj(1.5,Ka)./rootKa;
h0p_ka=-spherFactor*besselh(1.5,1,Ka)./rootKa;
j0_kpa=spherFactor*besselj(0.5,Kpa)./rootKpa;
j0p_kpa=-spherFactor*besselj(1.5,Kpa)./rootKpa;

scattCoeff(:,:,1)=-(q.*j0p_ka.*j0_kpa-j0_ka.*j0p_kpa)./...
    (q.*h0p_ka.*j0_kpa-h0_ka.*j0p_kpa);
%save the Bessel/Hankel functions for use in the next recurrence relation
jn_1ka=j0_ka;
hn_1ka=h0_ka;
jn_1kpa=j0_kpa;

for n=1:nMax+1
    jn_ka=spherFactor*besselj(n+0.5,Ka)./rootKa;
    hn_ka=spherFactor*besselh(n+0.5,1,Ka)./rootKa;
    jn_kpa=spherFactor*besselj(n+0.5,Kpa)./rootKpa;
    
    %Use recurrence relation for Bessel/Hankel functions
    jnp_ka=jn_1ka-(n+1)*jn_ka./Ka;
    hnp_ka=hn_1ka-(n+1).*hn_ka./Ka;
    jnp_kpa=jn_1kpa-(n+1)*jn_kpa./Kpa;
    
    scattCoeff(:,:,n+1)=-(q.*jnp_ka.*jn_kpa-jn_ka.*jnp_kpa)./...
    (q.*hnp_ka.*jn_kpa-hn_ka.*jnp_kpa);

% save the Bessel Hankel functions for use in the next recurrence relation
    jn_1ka=jn_ka;
    hn_1ka=hn_ka;
    jn_1kpa=jn_kpa;
end
 
end


