function [gammaCT]=thermalFactors(tempK,physProps,K,freq)
%calculates the thermal factors for compressional and thermal modes
%to relate the velocity potential of the field to the temperature field
%given the temperature, physical properties, wavenumbers and frequency
%Use the A&H version of thermal factors, divided by iomega.
%A&H have an extra term which is important for solids, negligible for
%liquids
%
%inputs are temperature, structure array physProps 
%the wavenumbers in all susbtances for each mode at each frequency
%and a row vector of frequency values
%
%{ 
physProps has the following fields:
handles.properties.physProps{1}.subsName='water';
handles.properties.physProps{1}.subsDesc='default water properties';
handles.properties.physProps{1}.matType; 'solid','liquid' or 'cavity'
handles.properties.physProps{1}.soundVel=1509.127; %m/s
handles.properties.physProps{1}.density=995.65; %kg/m3
handles.properties.physProps{1}.shearViscMod=0.0007973; %shear viscosity (lquids) or shear modulus Pa s
handles.properties.physProps{1}.compressMod=; %compressibility (liquids) or compression modulus
handles.properties.physProps{1}.conduct=0.6032; %J/m s K (thermal conductivity)
handles.properties.physProps{1}.heatCapCp=4178.2; %J/kg K
handles.properties.physProps{1}.expans=0.000301; %/K
handles.properties.physProps{1}.alphaFac=0.022;
handles.properties.physProps{1}.alphaExp=2.0;
%}
%output is an array of thermal factors for compressional and thermal modes
%in each substance included in the properties data
%gammaCT(substance,mode,freq)

numSubs=size(physProps,2); %second dimension is number of substances
gammaCT=zeros(numSubs,2,length(freq));

for iSubs=1:numSubs
    if ~strcmpi(physProps{iSubs}.matType,'cavity') %not a cavity
    kfreq=squeeze(K(iSubs,1,:)); %k for first mode as function of frequency
    lfreq=squeeze(K(iSubs,2,:)); %L for thermal mode as function of frequency
    unitArray=ones(size(freq))'; %an array same size as freq array
    omega=2*pi*1.0e6*freq'; %angular freq as a column (freq is in MHz)

    rho=physProps{iSubs}.density;
    c=physProps{iSubs}.soundVel; %sound speed as freq-array

    %now calculate other sound speed (for solids)
    %cl=sqrt((lambda+2/3mu)/rho), c=sqrt((lambda+2mu)/rho)
    
    %shear modulus or viscosity as an array like the freq array
    if strcmpi(physProps{iSubs}.matType,'solid')
        mu=physProps{iSubs}.shearViscMod.*unitArray;
        cl2=c*c*unitArray-4*mu/(3*rho);
        bulkMod=cl2*rho; % assumed adiabatic
    else %liquid
        mu=physProps{iSubs}.shearViscMod.*(-i*omega);
        cl2=c*c*unitArray;  %to avoid imaginary part to the velocity in liquids
        bulkMod=cl2*rho;
    end
    
    %ratio of heat capacities (gamma)
    %(see R N Thurston: Wave propagation in fluids and normal solids 
    % in W P Mason, Physical acoustics, Vol 1 part A, 1964 Academic Press, 
    %New York, p. 45)
    ratioCpCv=1+tempK*physProps{iSubs}.expans*physProps{iSubs}.expans*...
        bulkMod/(rho*physProps{iSubs}.heatCapCp); %if adiabatic bulk modulus

%    ratioCpCv=1/(1-tempK*physProps{iSubs}.expans*physProps{iSubs}.expans*...
%        bulkMod/(rho*physProps{iSubs}.heatCapCp)); %if isothermal bulk modulus

%the thermal factors for solids:  (from Allegra and Hawley)
%(for liquids, divide by -iomega (but not necessary - a common factor))
    gammaCT(iSubs,1,:)=-(ratioCpCv./(cl2*physProps{iSubs}.expans)).*...
            (omega.*omega-((cl2./ratioCpCv)+4*mu/(3*rho)).*kfreq.*kfreq);     %compressional (propagational) mode
    gammaCT(iSubs,2,:)=-(ratioCpCv./(cl2*physProps{iSubs}.expans)).*...
            (omega.*omega-(cl2./ratioCpCv+4*mu/(3*rho)).*lfreq.*lfreq);     %thermal mode
%{   
% Previous results (below) : Harlen/Sleeman et al are invalid, but still numerically OK for fluids 
%(the equation can not be solved independently for each wave mode)
%This thermal factor was however given by RN Thurston in Mason Vol 1 ref
%above but this was when no thermal modes present
    %thermal diffusivity
    diffus=physProps{iSubs}.conduct/...
        (physProps{iSubs}.density*physProps{iSubs}.heatCapCp);
%these are the uppercase gammac and gammat

    gammaCT(iSubs,1,:)=-i*kfreq.*kfreq*(ratioCpCv-1)./...
            (physProps{iSubs}.expans*(omega+(i*ratioCpCv*diffus*...
            kfreq.*kfreq)));     %compressional (propagational) mode
    gammaCT(iSubs,2,:)=-i*lfreq.*lfreq*(ratioCpCv-1)./...
            (physProps{iSubs}.expans*(omega+(i*ratioCpCv*diffus*...
            lfreq.*lfreq)));      %thermal mode   

%}
    end
end

